@@ -2,6 +2,16 @@
 
 set -eu
 
+remove_externally_managed_marker()
+{
+    "${python_interpreter}" -c '
+import pathlib
+import sysconfig
+path = pathlib.Path(sysconfig.get_path("stdlib")) / "EXTERNALLY-MANAGED"
+path.unlink(missing_ok=True)
+'
+}
+
 install_ssh_keys()
 {
     if [ ! -f "${ssh_private_key_path}" ]; then
@@ -43,17 +53,14 @@ customize_bashrc()
     fi
 
     # Improve shell prompts for interactive use.
-    echo "export PS1='\[\e]0;\u@\h: \w\a\]\[\033[01;32m\]\u@\h\[\033[00m\]:\[\033[01;34m\]\w\[\033[00m\]\$ '" >> ~/.bashrc
+    echo "export PS1='"'\[\e]0;\u@\h: \w\a\]\[\033[01;32m\]\u@\h\[\033[00m\]:\[\033[01;34m\]\w\[\033[00m\]\$ '"'" >> ~/.bashrc
 }
 
 install_pip() {
     if ! "${python_interpreter}" -m pip.__main__ --version --disable-pip-version-check 2>/dev/null; then
         case "${python_version}" in
-            "2.7")
-                pip_bootstrap_url="https://ci-files.testing.ansible.com/ansible-test/get-pip-20.3.4.py"
-                ;;
             *)
-                pip_bootstrap_url="https://ci-files.testing.ansible.com/ansible-test/get-pip-23.1.2.py"
+                pip_bootstrap_url="https://ci-files.testing.ansible.com/ansible-test/get-pip-24.0.py"
                 ;;
         esac
 
@@ -68,18 +75,6 @@ install_pip() {
     fi
 }
 
-pip_install() {
-    pip_packages="$1"
-
-    while true; do
-        # shellcheck disable=SC2086
-        "${python_interpreter}" -m pip install --disable-pip-version-check ${pip_packages} \
-        && break
-        echo "Failed to install packages. Sleeping before trying again..."
-        sleep 10
-    done
-}
-
 bootstrap_remote_alpine()
 {
     py_pkg_prefix="py3"
@@ -168,19 +163,39 @@ bootstrap_remote_freebsd()
         jinja2_pkg="py${python_package_version}-jinja2"
         cryptography_pkg="py${python_package_version}-cryptography"
         pyyaml_pkg="py${python_package_version}-yaml"
+        packaging_pkg="py${python_package_version}-packaging"
 
         # Declare platform/python version combinations which do not have supporting OS packages available.
         # For these combinations ansible-test will use pip to install the requirements instead.
         case "${platform_version}/${python_version}" in
-            *)
+            13.3/3.9)
+                # defaults above 'just work'TM
+                ;;
+            13.3/3.11)
                 jinja2_pkg=""  # not available
                 cryptography_pkg=""  # not available
                 pyyaml_pkg=""  # not available
                 ;;
+            14.1/3.9)
+                # defaults above 'just work'TM
+                ;;
+            14.1/3.11)
+                cryptography_pkg=""  # not available
+                jinja2_pkg=""  # not available
+                pyyaml_pkg=""  # not available
+                ;;
+            *)
+                # just assume nothing is available
+                jinja2_pkg=""  # not available
+                cryptography_pkg=""  # not available
+                pyyaml_pkg=""  # not available
+                packaging_pkg="" # not available
+                ;;
         esac
 
         packages="
             ${packages}
+            ${packaging_pkg}
             libyaml
             ${pyyaml_pkg}
             ${jinja2_pkg}
@@ -242,79 +257,6 @@ bootstrap_remote_macos()
     echo 'PATH="/usr/local/bin:$PATH"' > /etc/zshenv
 }
 
-bootstrap_remote_rhel_7()
-{
-    packages="
-        gcc
-        python-devel
-        python-virtualenv
-        "
-
-    while true; do
-        # shellcheck disable=SC2086
-        yum install -q -y ${packages} \
-        && break
-        echo "Failed to install packages. Sleeping before trying again..."
-        sleep 10
-    done
-
-    install_pip
-
-    bootstrap_remote_rhel_pinned_pip_packages
-}
-
-bootstrap_remote_rhel_8()
-{
-    if [ "${python_version}" = "3.6" ]; then
-        py_pkg_prefix="python3"
-    else
-        py_pkg_prefix="python${python_version}"
-    fi
-
-    packages="
-        gcc
-        ${py_pkg_prefix}-devel
-        "
-
-    # pip isn't included in the Python devel package under Python 3.11
-    if [ "${python_version}" != "3.6" ]; then
-        packages="
-            ${packages}
-            ${py_pkg_prefix}-pip
-        "
-    fi
-
-    # Jinja2 is not installed with an OS package since the provided version is too old.
-    # Instead, ansible-test will install it using pip.
-    if [ "${controller}" ]; then
-        packages="
-            ${packages}
-            ${py_pkg_prefix}-cryptography
-            "
-    fi
-
-    # Python 3.11 isn't a module like the earlier versions
-    if [ "${python_version}" = "3.6" ]; then
-        while true; do
-            # shellcheck disable=SC2086
-            yum module install -q -y "python${python_package_version}" \
-            && break
-            echo "Failed to install packages. Sleeping before trying again..."
-            sleep 10
-        done
-    fi
-
-    while true; do
-        # shellcheck disable=SC2086
-        yum install -q -y ${packages} \
-        && break
-        echo "Failed to install packages. Sleeping before trying again..."
-        sleep 10
-    done
-
-    bootstrap_remote_rhel_pinned_pip_packages
-}
-
 bootstrap_remote_rhel_9()
 {
     if [ "${python_version}" = "3.9" ]; then
@@ -326,19 +268,12 @@ bootstrap_remote_rhel_9()
     packages="
         gcc
         ${py_pkg_prefix}-devel
+        ${py_pkg_prefix}-pip
         "
 
-    # pip is not included in the Python devel package under Python 3.11
-    if [ "${python_version}" != "3.9" ]; then
-        packages="
-            ${packages}
-            ${py_pkg_prefix}-pip
-        "
-    fi
-
     # Jinja2 is not installed with an OS package since the provided version is too old.
     # Instead, ansible-test will install it using pip.
-    # packaging and resolvelib are missing for Python 3.11 (and possible later) so we just
+    # packaging and resolvelib are missing for controller supported Python versions, so we just
     # skip them and let ansible-test install them from PyPI.
     if [ "${controller}" ]; then
         packages="
@@ -357,26 +292,44 @@ bootstrap_remote_rhel_9()
     done
 }
 
+bootstrap_remote_rhel_10()
+{
+    py_pkg_prefix="python3"
+
+    packages="
+        gcc
+        ${py_pkg_prefix}-devel
+        ${py_pkg_prefix}-pip
+        "
+
+    if [ "${controller}" ]; then
+        packages="
+            ${packages}
+            ${py_pkg_prefix}-cryptography
+            ${py_pkg_prefix}-jinja2
+            ${py_pkg_prefix}-packaging
+            ${py_pkg_prefix}-pyyaml
+            ${py_pkg_prefix}-resolvelib
+            "
+    fi
+
+    while true; do
+        # shellcheck disable=SC2086
+        dnf install -q -y ${packages} \
+        && break
+        echo "Failed to install packages. Sleeping before trying again..."
+        sleep 10
+    done
+}
+
 bootstrap_remote_rhel()
 {
     case "${platform_version}" in
-        7.*) bootstrap_remote_rhel_7 ;;
-        8.*) bootstrap_remote_rhel_8 ;;
         9.*) bootstrap_remote_rhel_9 ;;
+        10.*) bootstrap_remote_rhel_10 ;;
     esac
 }
 
-bootstrap_remote_rhel_pinned_pip_packages()
-{
-    # pin packaging and pyparsing to match the downstream vendored versions
-    pip_packages="
-        packaging==20.4
-        pyparsing==2.4.7
-        "
-
-    pip_install "${pip_packages}"
-}
-
 bootstrap_remote_ubuntu()
 {
     py_pkg_prefix="python3"
@@ -400,10 +353,6 @@ bootstrap_remote_ubuntu()
         # For these ansible-test will use pip to install the requirements instead.
         # Only the platform is checked since Ubuntu shares Python packages across Python versions.
         case "${platform_version}" in
-            "20.04")
-                jinja2_pkg=""  # too old
-                resolvelib_pkg=""  # not available
-                ;;
         esac
 
         packages="
@@ -430,14 +379,27 @@ bootstrap_docker()
 {
     # Required for newer mysql-server packages to install/upgrade on Ubuntu 16.04.
     rm -f /usr/sbin/policy-rc.d
+
+    for key_value in ${python_interpreters}; do
+        IFS=':' read -r python_version python_interpreter << EOF
+${key_value}
+EOF
+
+        echo "Bootstrapping Python ${python_version} at: ${python_interpreter}"
+
+        remove_externally_managed_marker
+    done
 }
 
 bootstrap_remote()
 {
-    for python_version in ${python_versions}; do
-        echo "Bootstrapping Python ${python_version}"
+    for key_value in ${python_interpreters}; do
+        IFS=':' read -r python_version python_interpreter << EOF
+${key_value}
+EOF
+
+        echo "Bootstrapping Python ${python_version} at: ${python_interpreter}"
 
-        python_interpreter="python${python_version}"
         python_package_version="$(echo "${python_version}" | tr -d '.')"
 
         case "${platform}" in
@@ -448,6 +410,8 @@ bootstrap_remote()
             "rhel") bootstrap_remote_rhel ;;
             "ubuntu") bootstrap_remote_ubuntu ;;
         esac
+
+        remove_externally_managed_marker
     done
 }
 
@@ -474,7 +438,7 @@ bootstrap_type=#{bootstrap_type}
 controller=#{controller}
 platform=#{platform}
 platform_version=#{platform_version}
-python_versions=#{python_versions}
+python_interpreters=#{python_interpreters}
 ssh_key_type=#{ssh_key_type}
 ssh_private_key=#{ssh_private_key}
 ssh_public_key=#{ssh_public_key}
